@@ -8,7 +8,8 @@
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the Free
-   Software Foundation, version 2.1 of the License.
+   Software Foundation; either version 2.1, or (at your option) any later
+   version.
 
    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -16,14 +17,13 @@
    for more details.
 
    You should have received a copy of the GNU Lesser General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef __GSSIP_H
 
 #define __GSSIP_H
 
-void init_gssip();
+void init_gssip(void);
 
 #endif
