@@ -32,9 +32,19 @@
 
 #include <pthread.h>
 
+#ifdef USE_DLOPEN
+#define SPD_AUDIO_PLUGIN_ENTRY spd_audio_plugin_get
+#else
 #define SPD_AUDIO_PLUGIN_ENTRY spd_nas_LTX_spd_audio_plugin_get
+#endif
 #include <spd_audio_plugin.h>
 
+#include "../common/common.h"
+
+/* Put a message into the logfile (stderr) */
+#define MSG(level, arg, ...) if (level <= nas_log_level) { MSG(0, "nas: " arg, ##__VA_ARGS__); }
+#define ERR(arg, ...) MSG(0, "nas ERROR: " arg, ##__VA_ARGS__)
+
 typedef struct {
 	AudioID id;
 	AuServer *aud;
@@ -63,23 +73,22 @@ static void *_nas_handle_events(void *par)
    since this handler gets called in the event handler thread. */
 static AuBool _nas_handle_server_error(AuServer * server, AuErrorEvent * event)
 {
-	fprintf(stderr, "ERROR: Non-fatal server error in NAS\n");
+	ERR("Non-fatal server error in NAS\n");
 
 	if (event->type != 0) {
-		fprintf(stderr,
-			"Event of a different type received in NAS error handler.");
+		ERR("Event of a different type received in NAS error handler.");
 		return -1;
 	}
 
 	/* It's a pain but we can't ask for string return code
 	   since it's not allowed to talk to the server inside error handlers
 	   because of possible deadlocks. */
-	fprintf(stderr, "NAS: Serial number of failed request: %d\n",
-		event->serial);
-	fprintf(stderr, "NAS: Error code: %d\n", event->error_code);
-	fprintf(stderr, "NAS: Resource id: %d\n", event->resourceid);
-	fprintf(stderr, "NAS: Request code: %d\n", event->request_code);
-	fprintf(stderr, "NAS: Minor code: %d\n\n", event->minor_code);
+	ERR("NAS: Serial number of failed request: %d\n",
+		(int) event->serial);
+	ERR("NAS: Error code: %d\n", event->error_code);
+	ERR("NAS: Resource id: %d\n", (int) event->resourceid);
+	ERR("NAS: Request code: %d\n", event->request_code);
+	ERR("NAS: Minor code: %d\n\n", event->minor_code);
 
 	return 0;
 }
@@ -88,19 +97,21 @@ static AudioID *nas_open(void **pars)
 {
 	spd_nas_id_t *nas_id;
 	int ret;
+	AuServer *aud;
 
-	nas_id = (spd_nas_id_t *) g_malloc(sizeof(spd_nas_id_t));
-
-	nas_id->aud = AuOpenServer(pars[2], 0, NULL, 0, NULL, NULL);
-	if (!nas_id->aud) {
-		fprintf(stderr, "Can't connect to NAS audio server\n");
+	aud = AuOpenServer(pars[2], 0, NULL, 0, NULL, NULL);
+	if (!aud) {
+		ERR("Can't connect to NAS audio server\n");
 		return NULL;
 	}
 
+	nas_id = (spd_nas_id_t *) g_malloc(sizeof(spd_nas_id_t));
+	nas_id->aud = aud;
+
 	AuSetErrorHandler(nas_id->aud, _nas_handle_server_error);
 	/* return value incompatible with documentation here */
 	/*    if (!r){
-	   fprintf(stderr, "Can't set default NAS event handler\n");
+	   ERR("Can't set default NAS event handler\n");
 	   return -1;
 	   } */
 
@@ -114,8 +125,7 @@ static AudioID *nas_open(void **pars)
 	    pthread_create(&nas_id->nas_event_handler, NULL, _nas_handle_events,
 			   (void *)nas_id);
 	if (ret != 0) {
-		fprintf(stderr,
-			"ERROR: NAS Audio module: thread creation failed\n");
+		ERR("NAS Audio module: thread creation failed\n");
 		return NULL;
 	}
 
@@ -127,7 +137,7 @@ static int nas_play(AudioID * id, AudioTrack track)
 	char *buf;
 	Sound s;
 	AuEventHandlerRec *event_handler;
-	float lenght;
+	float length;
 	struct timeval now;
 	struct timespec timeout;
 	spd_nas_id_t *nas_id = (spd_nas_id_t *) id;
@@ -154,23 +164,22 @@ static int nas_play(AudioID * id, AudioTrack track)
 
 	if (event_handler == NULL) {
 		pthread_mutex_unlock(&nas_id->flow_mutex);
-		fprintf(stderr,
-			"AuSoundPlayFromData failed for unknown resons.\n");
+		ERR("AuSoundPlayFromData failed for unknown reasons.\n");
 		return -1;
 	}
 
 	if (nas_id->flow == 0) {
-		fprintf(stderr, "Couldn't start data flow");
+		ERR("Couldn't start data flow");
 	}
 	pthread_mutex_unlock(&nas_id->flow_mutex);
 
 	/* Another timing magic */
 	pthread_mutex_lock(&nas_id->pt_mutex);
-	lenght = (((float)track.num_samples) / (float)track.sample_rate);
+	length = (((float)track.num_samples) / (float)track.sample_rate);
 	gettimeofday(&now, NULL);
-	timeout.tv_sec = now.tv_sec + (int)lenght;
+	timeout.tv_sec = now.tv_sec + (int)length;
 	timeout.tv_nsec =
-	    now.tv_usec * 1000 + (lenght - (int)lenght) * 1000000000;
+	    now.tv_usec * 1000 + (length - (int)length) * 1000000000;
 	pthread_cond_timedwait(&nas_id->pt_cond, &nas_id->pt_mutex, &timeout);
 	pthread_mutex_unlock(&nas_id->pt_mutex);
 
@@ -256,5 +265,9 @@ spd_audio_plugin_t *nas_plugin_get(void)
 	return &nas_functions;
 }
 
-spd_audio_plugin_t *SPD_AUDIO_PLUGIN_ENTRY(void)
-    __attribute__ ((weak, alias("nas_plugin_get")));
+spd_audio_plugin_t *
+    __attribute__ ((weak))
+    SPD_AUDIO_PLUGIN_ENTRY(void)
+{
+	return &nas_functions;
+}
