@@ -24,9 +24,8 @@
 
 #define __SPDSEND_H
 
-#define _GNU_SOURCE
-
 #include <stddef.h>
+#include <stdbool.h>
 
 /* Configuration */
 
@@ -36,7 +35,6 @@
 
 /* Types */
 
-typedef enum { FALSE, TRUE } bool;
 typedef int Stream;
 typedef int Connection_Id;
 
