@@ -2,7 +2,7 @@
  * baratinoo.c - Speech Dispatcher backend for Baratinoo (VoxyGen)
  *
  * Copyright (C) 2016 Brailcom, o.p.s.
- * Copyright (C) 2019-2020 Samuel Thibault <samuel.thibault@ens-lyon.org>
+ * Copyright (C) 2019-2021 Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  *
  * - The input is sent to the engine through a BCinputTextBuffer.  There is
  *   a single one of those at any given time, and it is filled in
- *   module_speak() and consumed in the synthesis thread.
+ *   module_speak_sync().
  *
  *   This doesn't use an input callback generating a continuous flow (and
  *   blocking waiting for more data) even though it would be a fairly nice
@@ -51,10 +51,6 @@
 #include <config.h>
 #endif
 
-#include <semaphore.h>
-
-#include "module_utils_speak_queue.h"
-
 #ifdef BARATINOO_ABI_IS_STABLE_ENOUGH_FOR_ME
 /* See below why this is problematic.  It can however be useful to get the
  * compiler help to check compatibility */
@@ -179,32 +175,24 @@ static const bv_BCinputTextBufferNew_t bv_BCinputTextBufferNew[] = {
 
 /*------------------------ Speech-Dispatcher module ------------------------*/
 
-#include "spd_audio.h"
-
 #include <speechd_types.h>
 
 #include "module_utils.h"
 
 #define MODULE_NAME     "baratinoo"
 #define DBG_MODNAME     "Baratinoo: "
-#define MODULE_VERSION  "0.1"
+#define MODULE_VERSION  "0.2"
 
 #define DEBUG_MODULE 1
 DECLARE_DEBUG();
 
 typedef struct {
-	/* Thread primitives */
-	pthread_t thread;
-	sem_t semaphore;
-
 #ifndef BARATINOO_ABI_IS_STABLE_ENOUGH_FOR_ME
 	SupportedBaratinooVersion supported_version;
 #endif
 
 	BCengine engine;
-	/* The buffer consumed by the TTS engine.  It is NULL when the TTS
-	 * thread is ready to accept new input.  Otherwise, the thread is in
-	 * the process of synthesizing speech. */
+	/* The buffer consumed by the TTS engine. */
 	BCinputTextBuffer buffer;
 
 	SPDVoice **voice_list;
@@ -213,7 +201,9 @@ typedef struct {
 	int voice;
 
 	/* request flags */
-	gboolean close_requested;
+	gboolean stop_requested;
+	gboolean pause_requested;
+	gboolean pause_index_sent;
 } Engine;
 
 /* engine and state */
@@ -222,11 +212,14 @@ static Engine baratinoo_engine = {
 	.buffer = NULL,
 	.voice_list = NULL,
 	.voice = 0,
-	.close_requested = FALSE
+	.stop_requested = FALSE,
+	.pause_requested = FALSE,
+	.pause_index_sent = FALSE,
 };
 
+static gboolean BC_initialized = FALSE;
+
 /* Internal functions prototypes */
-static void *_baratinoo_speak(void *);
 static SPDVoice **baratinoo_list_voices(BCengine *engine);
 /* Parameters */
 static void baratinoo_set_voice_type(SPDVoiceType voice);
@@ -315,11 +308,11 @@ static SupportedBaratinooVersion get_baratinoo_supported_version(void)
 int module_init(char **status_info)
 {
 	Engine *engine = &baratinoo_engine;
-	int ret;
 	BARATINOOC_STATE state;
 
 	DBG(DBG_MODNAME "Module init");
-	INIT_INDEX_MARKING();
+
+	module_audio_set_server();
 
 	DBG(DBG_MODNAME "BaratinooPunctuationList = %s", BaratinooPunctuationList);
 	DBG(DBG_MODNAME "BaratinooIntonationList = %s", BaratinooIntonationList);
@@ -327,8 +320,6 @@ int module_init(char **status_info)
 
 	*status_info = NULL;
 
-	engine->close_requested = FALSE;
-
 	/* Init Baratinoo */
 	if (BCinitlib(baratinoo_trace_cb) != BARATINOO_INIT_OK) {
 		DBG(DBG_MODNAME "Failed to initialize library");
@@ -337,6 +328,7 @@ int module_init(char **status_info)
 					"properly set up.");
 		return -1;
 	}
+	BC_initialized = TRUE;
 	DBG(DBG_MODNAME "Using Baratinoo %s", BCgetBaratinooVersion());
 
 #ifndef BARATINOO_ABI_IS_STABLE_ENOUGH_FOR_ME
@@ -387,26 +379,6 @@ int module_init(char **status_info)
 
 	BCsetWantedEvent(engine->engine, BARATINOO_MARKER_EVENT);
 
-	/* Setup TTS thread */
-	sem_init(&engine->semaphore, 0, 0);
-
-	DBG(DBG_MODNAME "creating new thread for baratinoo_speak");
-	ret = pthread_create(&engine->thread, NULL, _baratinoo_speak, engine);
-	if (ret != 0) {
-		DBG(DBG_MODNAME "thread creation failed");
-		*status_info =
-		    g_strdup("The module couldn't initialize threads. "
-			     "This could be either an internal problem or an "
-			     "architecture problem. If you are sure your architecture "
-			     "supports threads, please report a bug.");
-		return -1;
-	}
-
-	if (module_speak_queue_init(BaratinooQueueSize, status_info)) {
-		DBG(DBG_MODNAME "queue creation failed");
-		return -1;
-	}
-
 	DBG(DBG_MODNAME "Initialization successfully.");
 	*status_info = g_strdup("Baratinoo initialized successfully.");
 
@@ -420,7 +392,7 @@ SPDVoice **module_list_voices(void)
 	return engine->voice_list;
 }
 
-int module_speak(gchar *data, size_t bytes, SPDMessageType msgtype)
+void module_speak_sync(const gchar *data, size_t bytes, SPDMessageType msgtype)
 {
 	Engine *engine = &baratinoo_engine;
 	GString *buffer = NULL;
@@ -435,15 +407,13 @@ int module_speak(gchar *data, size_t bytes, SPDMessageType msgtype)
 
 	if (engine->buffer != NULL) {
 		DBG(DBG_MODNAME "WARNING: module_speak() called during speech");
-		return 0;
+		module_speak_error();
+		return;
 	}
 
 	/* select voice following parameters.  we don't use tags for this as
 	 * we need to do some computation on our end anyway and need pass an
 	 * ID when creating the buffer too */
-	/* NOTE: these functions access the engine, which wouldn't be safe if
-	 *       we didn't know that the thread is sleeping.  But we do know it
-	 *       is, as @c Engine::buffer is NULL */
 	UPDATE_STRING_PARAMETER(voice.language, baratinoo_set_language);
 	UPDATE_PARAMETER(voice_type, baratinoo_set_voice_type);
 	UPDATE_STRING_PARAMETER(voice.name, baratinoo_set_synthesis_voice);
@@ -452,6 +422,7 @@ int module_speak(gchar *data, size_t bytes, SPDMessageType msgtype)
 					      BARATINOO_UTF8, engine->voice, 0);
 	if (!engine->buffer) {
 		DBG(DBG_MODNAME "Failed to allocate input buffer");
+		module_speak_error();
 		goto err;
 	}
 
@@ -507,15 +478,68 @@ int module_speak(gchar *data, size_t bytes, SPDMessageType msgtype)
 	DBG(DBG_MODNAME "Sending buffer: %s", buffer->str);
 	if (!BCinputTextBufferInit(engine->buffer, buffer->str)) {
 		DBG(DBG_MODNAME "Failed to initialize input buffer");
+		module_speak_error();
 		goto err;
 	}
 
 	g_string_free(buffer, TRUE);
+	buffer = NULL;
 
-	sem_post(&engine->semaphore);
+	engine->stop_requested = FALSE;
+	engine->pause_requested = FALSE;
+	engine->pause_index_sent = FALSE;
 
-	DBG(DBG_MODNAME "leaving module_speak() normally");
-	return bytes;
+	BARATINOOC_STATE state = BARATINOO_READY;
+
+	state = BCinputTextBufferSetInEngine(engine->buffer, engine->engine);
+	if (state != BARATINOO_READY) {
+		DBG(DBG_MODNAME "Failed to set input buffer");
+		module_speak_error();
+		goto out;
+	}
+
+	module_speak_ok();
+
+	module_report_event_begin();
+	do {
+		if (engine->stop_requested || (engine->pause_requested && engine->pause_index_sent)) {
+			BCpurge(engine->engine);
+			engine->buffer = NULL;
+			break;
+		}
+
+		/* Process server events in case we were told to stop in between */
+		module_process(STDIN_FILENO, 0);
+
+		state = BCprocessLoop(engine->engine, BaratinooResponsiveness);
+		if (state == BARATINOO_EVENT) {
+			BaratinooEvent event = BCgetEvent(engine->engine);
+			if (event.type == BARATINOO_MARKER_EVENT) {
+				DBG(DBG_MODNAME "Reached mark '%s' at sample %lu", event.data.marker.name, event.sampleStamp);
+				module_report_index_mark(event.data.marker.name);
+				if (engine->pause_requested &&
+					!strncmp(event.data.marker.name,
+						INDEX_MARK_BODY,
+						INDEX_MARK_BODY_LEN)) {
+					engine->pause_index_sent = 1;
+				}
+			}
+		}
+	} while (state == BARATINOO_RUNNING || state == BARATINOO_EVENT);
+
+out:
+	if (engine->pause_requested)
+		module_report_event_pause();
+	else if (engine->stop_requested)
+		module_report_event_stop();
+	else
+		module_report_event_end();
+
+	BCinputTextBufferDelete(engine->buffer);
+	engine->buffer = NULL;
+
+	DBG(DBG_MODNAME "leaving module_speak_sync() normally");
+	return;
 
 err:
 	if (buffer)
@@ -525,21 +549,25 @@ err:
 		engine->buffer = NULL;
 	}
 
-	return 0;
+	return;
 }
 
 int module_stop(void)
 {
+	Engine *engine = &baratinoo_engine;
+
 	DBG(DBG_MODNAME "Stop requested");
-	module_speak_queue_stop();
+	engine->stop_requested = TRUE;
 
 	return 0;
 }
 
 size_t module_pause(void)
 {
+	Engine *engine = &baratinoo_engine;
+
 	DBG(DBG_MODNAME "Pause requested");
-	module_speak_queue_pause();
+	engine->stop_requested = TRUE;
 
 	return 0;
 }
@@ -550,24 +578,6 @@ int module_close(void)
 
 	DBG(DBG_MODNAME "close()");
 
-	DBG(DBG_MODNAME "Terminating threads");
-
-	module_speak_queue_terminate();
-
-	/* Politely ask the thread to terminate */
-	engine->close_requested = TRUE;
-	sem_post(&engine->semaphore);
-	/* ...and give it a chance to actually quit. */
-	g_usleep(25000);
-
-	/* Make sure the thread has really exited */
-	pthread_cancel(engine->thread);
-	DBG(DBG_MODNAME "Joining threads.");
-	if (pthread_join(engine->thread, NULL) != 0)
-		DBG(DBG_MODNAME "Failed to join threads.");
-
-	sem_destroy(&engine->semaphore);
-
 	/* destroy voice list */
 	if (engine->voice_list != NULL) {
 		int i;
@@ -587,10 +597,11 @@ int module_close(void)
 	    engine->engine = NULL;
 	}
 
-	/* uninitialize */
-	BCterminatelib();
-
-	module_speak_queue_free();
+	if (BC_initialized) {
+		/* uninitialize */
+		BCterminatelib();
+		BC_initialized = FALSE;
+	}
 
 	DBG(DBG_MODNAME "Module closed.");
 
@@ -640,82 +651,6 @@ static SPDVoice **baratinoo_list_voices(BCengine *engine)
     return voices;
 }
 
-void module_speak_queue_cancel(void)
-{
-	/* We will stop the synth from _baratinoo_speak */
-}
-
-/**
- * @brief Internal TTS thread.
- * @param data An Engine structure.
- * @returns NULL.
- *
- * The TTS thread.  It waits on @c Engine::semaphore to consume input data
- * from @c Engine::buffer.
- *
- * @see Engine::close_requested
- */
-static void *_baratinoo_speak(void *data)
-{
-	Engine *engine = data;
-	BARATINOOC_STATE state = BARATINOO_READY;
-
-	set_speaking_thread_parameters();
-
-	while (!engine->close_requested) {
-		sem_wait(&engine->semaphore);
-		DBG(DBG_MODNAME "Semaphore on");
-
-		if (!engine->buffer)
-			continue;
-
-		state = BCinputTextBufferSetInEngine(engine->buffer, engine->engine);
-		if (state != BARATINOO_READY) {
-			DBG(DBG_MODNAME "Failed to set input buffer");
-			goto cont;
-		}
-
-		if (!module_speak_queue_before_synth())
-			goto cont;
-
-		if (module_speak_queue_stop_requested() || engine->close_requested) {
-			DBG(DBG_MODNAME "Stop in child, terminating");
-			goto cont;
-		}
-
-		module_speak_queue_before_play();
-		do {
-			state = BCprocessLoop(engine->engine, BaratinooResponsiveness);
-			if (state == BARATINOO_EVENT) {
-				BaratinooEvent event = BCgetEvent(engine->engine);
-				if (event.type == BARATINOO_MARKER_EVENT) {
-					DBG(DBG_MODNAME "Reached mark '%s' at sample %lu", event.data.marker.name, event.sampleStamp);
-					module_speak_queue_add_mark(event.data.marker.name);
-				}
-			} else if (state == BARATINOO_INPUT_ERROR ||
-				   state == BARATINOO_ENGINE_ERROR) {
-				module_speak_queue_stop();
-			}
-		} while (state == BARATINOO_RUNNING || state == BARATINOO_EVENT);
-
-		if (module_speak_queue_stop_requested() || engine->close_requested) {
-			DBG(DBG_MODNAME "Stop in child, terminating");
-		} else {
-			DBG(DBG_MODNAME "Finished synthesizing");
-			module_speak_queue_add_end();
-		}
-
-cont:
-		BCinputTextBufferDelete(engine->buffer);
-		engine->buffer = NULL;
-	}
-
-
-	DBG(DBG_MODNAME "leaving thread with state=%d", state);
-
-	pthread_exit(NULL);
-}
-
 /* Voice selection */
 
 /**
@@ -1005,15 +940,16 @@ static void baratinoo_trace_cb(BaratinooTraceLevel level, int engine_num, const
  */
 static int baratinoo_output_signal(void *private_data, const void *address, int length)
 {
+	Engine *engine = private_data;
+
 	/* If stop is requested during synthesis, abort here to stop speech as
 	 * early as possible, even if the engine didn't finish its cycle yet. */
-	if (module_speak_queue_stop_requested())
+	if (engine->stop_requested)
 	{
 		DBG(DBG_MODNAME "Not playing message because it got stopped");
 		return 1;
 	}
 
-	/* Engine *engine = private_data; */
 	AudioTrack track;
 #if defined(BYTE_ORDER) && (BYTE_ORDER == BIG_ENDIAN)
 	AudioFormat format = SPD_AUDIO_BE;
@@ -1029,9 +965,9 @@ static int baratinoo_output_signal(void *private_data, const void *address, int
 	track.samples = (short *) address;
 
 	DBG(DBG_MODNAME "Queueing %d samples", length / 2);
-	module_speak_queue_add_audio(&track, format);
+	module_tts_output_server(&track, format);
 
-	return module_speak_queue_stop_requested();
+	return engine->stop_requested;
 }
 
 /* SSML conversion functions */
