@@ -41,28 +41,10 @@
 #include "ivona_client.h"
 
 static struct sockaddr_in sinadr;
-char *ivona_get_wave_from_cache(char *to_say, int *nsamples);
-void ivona_store_wave_in_cache(char *to_say, char *wave, int nsamples);
 
-int ivona_init_sock(char *host, int port)
+static int ivona_new_sock(void)
 {
-	if (!inet_aton(host, &sinadr.sin_addr)) {
-		struct hostent *h = gethostbyname(host);
-		if (!h)
-			return -1;
-		memcpy(&sinadr.sin_addr, h->h_addr, sizeof(struct in_addr));
-		endhostent();
-	}
-	sinadr.sin_family = AF_INET;
-	sinadr.sin_port = htons(port);
-	return 0;
-}
 
-#define BASE_WAVE_SIZE 65536
-#define STEP_WAVE_SIZE 32768
-
-int ivona_send_string(char *to_say)
-{
 	int fd;
 
 	fd = socket(AF_INET, SOCK_STREAM, 0);
@@ -72,6 +54,38 @@ int ivona_send_string(char *to_say)
 		close(fd);
 		return -1;
 	}
+	return fd;
+}
+
+int ivona_init_sock(const char *host, int port)
+{
+	int fd;
+	if (!inet_aton(host, &sinadr.sin_addr)) {
+		struct hostent *h = gethostbyname(host);
+		if (!h)
+			return -1;
+		memcpy(&sinadr.sin_addr, h->h_addr, sizeof(struct in_addr));
+		endhostent();
+	}
+	sinadr.sin_family = AF_INET;
+	sinadr.sin_port = htons(port);
+	fd = ivona_new_sock();
+	if (fd < 0)
+		return -1;
+	close(fd);
+	return 0;
+}
+
+#define BASE_WAVE_SIZE 65536
+#define STEP_WAVE_SIZE 32768
+
+int ivona_send_string(const char *to_say)
+{
+	int fd = ivona_new_sock();
+
+	if (fd < 0)
+		return -1;
+
 	write(fd, to_say, strlen(to_say));
 	write(fd, "\n", 1);
 	return fd;
@@ -127,7 +141,7 @@ char *ivona_get_wave_fd(int fd, int *nsamples, int *offset)
 static char *ivona_get_wave_from_cache(char *to_say,int *nsamples);
 void ivona_store_wave_in_cache(char *to_say,char *wave,int nsamples);
 */
-char *ivona_get_wave(char *to_say, int *nsamples, int *offset)
+char *ivona_get_wave(const char *to_say, int *nsamples, int *offset)
 {
 	int fd;
 	char *s;
@@ -147,7 +161,7 @@ char *ivona_get_wave(char *to_say, int *nsamples, int *offset)
 
 /* Plays the specified audio file - from ibmtts/espeak module */
 
-void play_icon(char *path, char *name)
+void play_icon(const char *path, const char *name)
 {
 	char *buf = g_strdup_printf("%s/%s", path, name);
 	module_play_file(buf);
@@ -207,7 +221,7 @@ static struct ivona_cache *find_min_count(void)
 	return found;
 }
 
-void ivona_store_wave_in_cache(char *str, char *wave, int samples)
+void ivona_store_wave_in_cache(const char *str, const char *wave, int samples)
 {
 
 	struct ivona_cache *ica;
@@ -230,7 +244,7 @@ void ivona_store_wave_in_cache(char *str, char *wave, int samples)
 	DBG("Stored cache %s", str);
 }
 
-char *ivona_get_wave_from_cache(char *to_say, int *samples)
+char *ivona_get_wave_from_cache(const char *to_say, int *samples)
 {
 	struct ivona_cache *ica;
 	if (strlen(to_say) > IVONA_CACHE_MAX_STRLEN)
