@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 #ifndef IVONA_CLIENT_H
@@ -25,12 +23,12 @@
 /* A constant, used in both ivona.c and ivona_client.c */
 #define IVONA_CACHE_MAX_STRLEN 11
 
-int ivona_init_sock(char *host, int port);
-int ivona_send_string(char *to_say);
+int ivona_init_sock(const char *host, int port);
+int ivona_send_string(const char *to_say);
 char *ivona_get_wave_fd(int fd, int *nsamples, int *offset);
-char *ivona_get_wave(char *to_say, int *nsamples, int *offset);
-void play_icon(char *path, char *name);
+char *ivona_get_wave(const char *to_say, int *nsamples, int *offset);
+void play_icon(const char *path, const char *name);
 void ivona_init_cache(void);
-void ivona_store_wave_in_cache(char *to_say, char *wave, int nsamples);
-char *ivona_get_wave_from_cache(char *to_say, int *nsamples);
+void ivona_store_wave_in_cache(const char *to_say, const char *wave, int nsamples);
+char *ivona_get_wave_from_cache(const char *to_say, int *nsamples);
 #endif
