@@ -368,6 +368,14 @@ OutputModule *load_output_module(const char *mod_name, const char *mod_prog,
 	module->configdir = g_strdup(mod_cfg_dir);
 	module->stderr_redirect = -1;
 
+	pthread_mutex_init(&module->read_mutex, NULL);
+	pthread_cond_init(&module->reply_cond, NULL);
+	pthread_cond_init(&module->event_cond, NULL);
+	module->reply = NULL;
+	module->event = NULL;
+	module->reading_message = FALSE;
+	module->waiting_for_reply = FALSE;
+
 	if (module->progdir) {
 		module->filename = (char *)spd_get_path(mod_prog, module->progdir);
 	} else {
@@ -392,6 +400,7 @@ OutputModule *load_output_module(const char *mod_name, const char *mod_prog,
 			module_conf_dir = g_strdup_printf("%s/modules",
 							  SpeechdOptions.conf_dir);
 			MSG(4, "%s does not exist, looking in %s", module->configfilename, module_conf_dir);
+			g_free(module->configfilename);
 			module->configfilename =
 			    (char *)spd_get_path(mod_cfgfile, module_conf_dir);
 			g_free(module_conf_dir);
@@ -597,7 +606,7 @@ OutputModule *load_output_module(const char *mod_name, const char *mod_prog,
 	}
 
 	/* Try to get the list of voices */
-	SPDVoice **voices = output_get_voices(module);
+	SPDVoice **voices = output_get_voices(module, NULL, NULL);
 	if (!voices) {
 		/* No list of voices, that would surprise clients, let's give up
 		 * on this module */
